@@ -34,11 +34,12 @@
 
 #include "lime.h"
 
-int write_vaddr_tcp(void *, size_t);
+ssize_t write_vaddr_tcp(void *, size_t);
 int setup_tcp(void);
 void cleanup_tcp(void);
 
 extern int port;
+extern int localhostonly;
 
 static struct socket *control;
 static struct socket *accept;
@@ -66,7 +67,12 @@ int setup_tcp() {
 	
 	saddr.sin_family = AF_INET;
    	saddr.sin_port = htons(port);
-	saddr.sin_addr.s_addr = INADDR_ANY;
+   	if (localhostonly) {
+   		saddr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+   	} else {
+		saddr.sin_addr.s_addr = htonl(INADDR_ANY);
+   	}
+	
 
 	fs = get_fs();
 	set_fs(KERNEL_DS);
@@ -124,27 +130,18 @@ void cleanup_tcp() {
 	}
 }
 
-int write_vaddr_tcp(void * v, size_t is) {
-	mm_segment_t fs;
+ssize_t write_vaddr_tcp(void * v, size_t is) {
+	ssize_t s;
+	struct kvec iov;
+	struct msghdr msg;
 
-	long s;
-
-	struct iovec iov;
-#if LINUX_VERSION_CODE < KERNEL_VERSION(3,19,0)
-	struct msghdr msg  = { .msg_iov = &iov, .msg_iovlen = 1 };
-#else
-	struct user_msghdr msg  = { .msg_iov = &iov, .msg_iovlen = 1 };
-#endif
-
-	fs = get_fs();
-	set_fs(KERNEL_DS);
+	memset(&iov, 0, sizeof(struct iovec));
+	memset(&msg, 0, sizeof(struct msghdr));
 
 	iov.iov_base = v;	
 	iov.iov_len = is;
     
-	s = sock_sendmsg(accept, &msg, is);
-
-	set_fs(fs);
+	s = kernel_sendmsg(accept, &msg, &iov, 1, is);
 
 	return s;
 }
